@@ -19,8 +19,8 @@
 ;;
 ;; You should have received a copy of the GNU General Public License along
 ;; with your copy of Emacs; see the file COPYING.  If not, write to the Free
-;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;; 02111-1307, USA.
+;; Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+;; MA 02110-1301, USA
 
 ;;; Commentary:
 ;;
@@ -34,6 +34,7 @@
 ;;; Code:
 
 (require 'compile)
+(require 'tramp)
 
 (defgroup pylint nil
   "Minor mode for running the Pylint Python checker"
