@@ -14,17 +14,24 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """Tkinker gui for pylint"""
+from __future__ import print_function
 
 import os
 import sys
 import re
-import Queue
 from threading import Thread
-from Tkinter import (Tk, Frame, Listbox, Entry, Label, Button, Scrollbar,
-                     Checkbutton, Radiobutton, IntVar, StringVar)
-from Tkinter import (TOP, LEFT, RIGHT, BOTTOM, END, X, Y, BOTH, SUNKEN, W,
-                     HORIZONTAL, DISABLED, NORMAL, W)
-from tkFileDialog import askopenfilename, askdirectory
+
+import six
+
+from six.moves.tkinter import (
+    Tk, Frame, Listbox, Entry, Label, Button, Scrollbar,
+    Checkbutton, Radiobutton, IntVar, StringVar,
+    TOP, LEFT, RIGHT, BOTTOM, END, X, Y, BOTH, SUNKEN, W,
+    HORIZONTAL, DISABLED, NORMAL, W,
+)
+from six.moves.tkinter_tkfiledialog import (
+    askopenfilename, askdirectory,
+)
 
 import pylint.lint
 from pylint.reporters.guireporter import GUIReporter
@@ -86,7 +93,7 @@ class BasicStream(object):
         """finalize what the contents of the dict should look like before output"""
         for item in self.outdict:
             num_empty = self.outdict[item].count('')
-            for _ in xrange(num_empty):
+            for _ in range(num_empty):
                 self.outdict[item].remove('')
             if self.outdict[item]:
                 self.outdict[item].pop(0)
@@ -97,7 +104,7 @@ class BasicStream(object):
         self.gui.tabs = self.outdict
         try:
             self.gui.rating.set(self.outdict['Global evaluation'][0])
-        except:
+        except KeyError:
             self.gui.rating.set('Error')
         self.gui.refresh_results_window()
 
@@ -118,7 +125,7 @@ class LintGui(object):
         #reporter
         self.reporter = None
         #message queue for output from reporter
-        self.msg_queue = Queue.Queue()
+        self.msg_queue = six.moves.queue.Queue()
         self.msgs = []
         self.visible_msgs = []
         self.filenames = []
@@ -171,10 +178,11 @@ class LintGui(object):
         rightscrollbar.pack(side=RIGHT, fill=Y)
         bottomscrollbar = Scrollbar(msg_frame, orient=HORIZONTAL)
         bottomscrollbar.pack(side=BOTTOM, fill=X)
-        self.lb_messages = Listbox(msg_frame,
-                  yscrollcommand=rightscrollbar.set,
-                  xscrollcommand=bottomscrollbar.set,
-                  bg="white")
+        self.lb_messages = Listbox(
+            msg_frame,
+            yscrollcommand=rightscrollbar.set,
+            xscrollcommand=bottomscrollbar.set,
+            bg="white")
         self.lb_messages.bind("<Double-Button-1>", self.show_sourcefile)
         self.lb_messages.pack(expand=True, fill=BOTH)
         rightscrollbar.config(command=self.lb_messages.yview)
@@ -185,10 +193,11 @@ class LintGui(object):
         rightscrollbar2.pack(side=RIGHT, fill=Y)
         bottomscrollbar2 = Scrollbar(history_frame, orient=HORIZONTAL)
         bottomscrollbar2.pack(side=BOTTOM, fill=X)
-        self.showhistory = Listbox(history_frame,
-                    yscrollcommand=rightscrollbar2.set,
-                    xscrollcommand=bottomscrollbar2.set,
-                    bg="white")
+        self.showhistory = Listbox(
+            history_frame,
+            yscrollcommand=rightscrollbar2.set,
+            xscrollcommand=bottomscrollbar2.set,
+            bg="white")
         self.showhistory.pack(expand=True, fill=BOTH)
         rightscrollbar2.config(command=self.showhistory.yview)
         bottomscrollbar2.config(command=self.showhistory.xview)
@@ -217,10 +226,11 @@ class LintGui(object):
         rightscrollbar.pack(side=RIGHT, fill=Y)
         bottomscrollbar = Scrollbar(res_frame, orient=HORIZONTAL)
         bottomscrollbar.pack(side=BOTTOM, fill=X)
-        self.results = Listbox(res_frame,
-                  yscrollcommand=rightscrollbar.set,
-                  xscrollcommand=bottomscrollbar.set,
-                  bg="white", font="Courier")
+        self.results = Listbox(
+            res_frame,
+            yscrollcommand=rightscrollbar.set,
+            xscrollcommand=bottomscrollbar.set,
+            bg="white", font="Courier")
         self.results.pack(expand=True, fill=BOTH, side=BOTTOM)
         rightscrollbar.config(command=self.results.yview)
         bottomscrollbar.config(command=self.results.xview)
@@ -318,7 +328,7 @@ class LintGui(object):
         self.txt_module.focus_set()
 
 
-    def select_recent_file(self, event):
+    def select_recent_file(self, event): # pylint: disable=unused-argument
         """adds the selected file in the history listbox to the Module box"""
         if not self.showhistory.size():
             return
@@ -349,7 +359,7 @@ class LintGui(object):
         try:
             for res in self.tabs[self.box.get()]:
                 self.results.insert(END, res)
-        except:
+        except KeyError:
             pass
 
     def process_incoming(self):
@@ -372,7 +382,7 @@ class LintGui(object):
                     fg_color = COLORS.get(msg_str[:3], 'black')
                     self.lb_messages.itemconfigure(END, fg=fg_color)
 
-            except Queue.Empty:
+            except six.moves.queue.Empty:
                 pass
         return True
 
@@ -392,15 +402,17 @@ class LintGui(object):
         """quit the application"""
         self.root.quit()
 
-    def halt(self):
+    def halt(self): # pylint: disable=no-self-use
         """program halt placeholder"""
         return
 
     def file_open(self, package=False, _=None):
         """launch a file browser"""
         if not package:
-            filename = askopenfilename(parent=self.root, filetypes=[('pythonfiles', '*.py'),
-                                                    ('allfiles', '*')], title='Select Module')
+            filename = askopenfilename(parent=self.root,
+                                       filetypes=[('pythonfiles', '*.py'),
+                                                  ('allfiles', '*')],
+                                       title='Select Module')
         else:
             filename = askdirectory(title="Select A Folder", mustexist=1)
 
@@ -471,7 +483,7 @@ class LintGui(object):
 
         self.root.configure(cursor='')
 
-    def show_sourcefile(self, event=None):
+    def show_sourcefile(self, event=None):  # pylint: disable=unused-argument
         selected = self.lb_messages.curselection()
         if not selected:
             return
@@ -498,7 +510,7 @@ def lint_thread(module, reporter, gui):
 def Run(args):
     """launch pylint gui from args"""
     if args:
-        print 'USAGE: pylint-gui\n launch a simple pylint gui using Tk'
+        print('USAGE: pylint-gui\n launch a simple pylint gui using Tk')
         sys.exit(1)
     gui = LintGui()
     gui.mainloop()
