@@ -1,4 +1,4 @@
-# pylint: disable=R0921
+# pylint: disable=R0921, print-statement
 """check assignment to function call where the function doesn't return
 
     'E1111': ('Assigning to function call which doesn\'t return',
@@ -28,6 +28,13 @@ def func_return_none():
 A = func_return_none()
 
 
+def func_implicit_return_none():
+    """Function returning None from bare return statement."""
+    return
+
+A = func_implicit_return_none()
+
+
 def func_return_none_and_smth():
     """function returning none and something else"""
     print 'dougloup'
